#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>

#include "WorkplaceWidget.h"
#include "ContactDockWidget.h"
#include "NavigationDockWidget.h"
#include "ChatDockWidget.h"

namespace miraclient
{

namespace gui
{

class LoginDialog;

class MainWindow : public QMainWindow
{
    Q_OBJECT

    public:
        MainWindow();
        virtual ~MainWindow();
        void setupTabs();

    public slots:
        void loginSuccessful();
        void loginFailed(const QString& error_message);
        void serverConnectionLost();
        void setActiveWorkplace(Workplace * wp,Utility * util = NULL);

    signals:
        void contactStateChanged(int contactIndex,int newState);
        void serverConnectionTerminated();

    protected:
        void closeEvent(QCloseEvent * event);

    private slots:
        void login();
        void about();
    private:
        void createActions();
        void createMenus();
        void createConnections();
        void createDockWidgets();
        void LoadSettings();
        void SaveSettings();
        WorkplaceWidget * m_workplaceWidget;
        ContactDockWidget * m_contactWidget;
        NavigationDockWidget * m_navigationWidget;
        ChatDockWidget * m_chatWidget;

        //Actions
        QAction * m_exitAction;
        QAction * m_aboutAction;
        QAction * m_loginAction;
        QAction * m_addContactAction;
        QAction * m_viewChatPaneAction;
        QAction * m_viewContactPaneAction;
        QAction * m_viewNavPaneAction;
        QAction * m_chatFocusAction;

        //Menus
        QMenu * m_fileMenu;
        QMenu * m_actionsMenu;
        QMenu * m_viewMenu;
        QMenu * m_helpMenu;
        QMenu * m_contactMenu;

        LoginDialog* m_loginDialog;
};
}//end gui namespace
}//end miraclient namespace
#endif // MAINWINDOW_H
