/*
 * AsioClient.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRACLIENT__NETWORK_ASIO_ASIOCLIENT_H
#define __MIRACLIENT__NETWORK_ASIO_ASIOCLIENT_H

#include <cstring>
#include <string>
#include <iostream>

#include <boost/asio.hpp>
#include <boost/thread/mutex.hpp>

#include "Client.h"
#include "Application.h"

namespace miraclient {

namespace network {

namespace asio {

#define INPUT_BUFFER_SIZE  1025 // 1 kilobyte (+1 for NULL char) for now.
#define OUTPUT_BUFFER_SIZE 1025 // 1 kilobyte (+1 '\n' char) for now.

class AsioClient : public Client
{
    public:
        AsioClient();
        virtual ~AsioClient() 
        {
            // TODO: finish processing messages in queue or delete them.

            Application::unset_client();
        }

        void connect(const std::string& hostname, const unsigned int port, const std::string& username, const std::string& password);

        size_t input_buffer_size()
        {
            return ((INPUT_BUFFER_SIZE-1) - m_input_buffer_size); // -1 because we want to leave a space for NULL char.
        }

        char* get_input_buffer()
        {
            return (m_input_buffer + m_input_buffer_size);
        }

        size_t output_buffer_size() { return m_output_buffer_size; }

        // NOTE: It is not necessary to pass a NULL character at the end of the string.
        //       Although doing so will not do any harm.
        void add_to_output_buffer(const char* msg, unsigned int size);

        void add_to_output_buffer(const char* msg)
        {
            add_to_output_buffer(msg, std::strlen(msg));
        }

        void add_to_output_buffer(const std::string& msg)
        {
            add_to_output_buffer(msg.c_str(), msg.length());
        }

        void send(const char* msg)
        {
            add_to_output_buffer(msg);
            std::cout << "[OUT]: " << msg << std::endl;
            send();
        }

        void send(const char* msg, unsigned int size)
        {
            add_to_output_buffer(msg, size);
            send();
        }

        void send(const std::string &msg)
        {
            add_to_output_buffer(msg);
            send();
        }

        void send();
        void receive();

        void disconnect() { delete this; }

    private:
        boost::asio::io_service      m_io;
        boost::asio::ip::tcp::socket m_socket;
        char                         m_input_buffer[INPUT_BUFFER_SIZE];
        size_t                       m_input_buffer_size;
        char                         m_output_buffer[OUTPUT_BUFFER_SIZE];
        size_t                       m_output_buffer_size;
static const size_t                  m_output_buffer_max_size;
        boost::mutex                 m_output_buffer_mutex;
        std::queue<std::string>     m_message_queue;
};

} // namespace asio

} // namespace network

} // namespace miraclient

#endif // __MIRACLIENT__NETWORK_ASIO_ASIOCLIENT_H
