#include "ContactDockWidget.h"
#include "User.h"
#include "MiraDefs.h"
using namespace miraclient::gui;
ContactDockWidget::ContactDockWidget()
{
    //ctor
    contactTree = new QTreeWidget;
    setWidget(contactTree);
    setWindowTitle(tr("Contacts"));
    setupUI();
    contactTree->expandAll();
}

ContactDockWidget::~ContactDockWidget()
{
    //dtor
    delete contactTree;
}

void ContactDockWidget::closeEvent(QCloseEvent * event)
{
    emit closed(false);
}

void ContactDockWidget::setupUI()
{
    QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(contactTree->sizePolicy().hasHeightForWidth());
    contactTree->setSizePolicy(sizePolicy);
    contactTree->setMinimumSize(QSize(100, 0));
    contactTree->headerItem()->setText(0, QString());
    contactTree->clear();

    activeTreeItem = new QTreeWidgetItem(contactTree);
    activeTreeItem->setText(0, tr("Active (1/5)"));

    onlineTreeItem = new QTreeWidgetItem(contactTree);
    onlineTreeItem->setText(0, tr("Online (2/5)"));

    offlineTreeItem = new QTreeWidgetItem(contactTree);
    offlineTreeItem->setText(0, tr("Offline (2/5)"));

    //Temporarily add users manually
    AddUser(QString("Max"), MIRA_CONTACTSTATE_ACTIVE);
    AddUser(QString("Thurston"), MIRA_CONTACTSTATE_OFFLINE);
    AddUser(QString("Daryl"), MIRA_CONTACTSTATE_IDLE);
    AddUser(QString("Alan"), MIRA_CONTACTSTATE_ONLINE);
}

void ContactDockWidget::UserStateChange()
{
}

void ContactDockWidget::AddUser(QString username, int state)
{
    QTreeWidgetItem * parentItem;
    QIcon * icon;
    if (state == MIRA_CONTACTSTATE_ACTIVE)
    {
        parentItem = activeTreeItem;
        icon = new QIcon(":/images/chat-active.png");
    }
    else if (state == MIRA_CONTACTSTATE_ONLINE)
    {
        parentItem = onlineTreeItem;
        icon = new QIcon(":/images/chat-online.png");
    }
    else if (state == MIRA_CONTACTSTATE_IDLE)
    {
        parentItem = onlineTreeItem;
        icon = new QIcon(":/images/chat-idle.png");
    }
    else if (state == MIRA_CONTACTSTATE_OFFLINE)
    {
        parentItem = offlineTreeItem;
        icon = new QIcon(":/images/chat-offline.png");
    }
    else
        //TODO: Create a custom exception to throw here
        throw;

    QTreeWidgetItem * __item = new QTreeWidgetItem(parentItem);
    __item->setText(0, username);
    __item->setIcon(0,*icon);
}
