#include <QtCore/QTranslator>
#include <QtCore/QLocale>
#include <QMessageBox>
#include "GuiApp.h"
#include "MiraDefs.h"
#include "Workplace.h"
#include "Utility.h"

using namespace miraclient::gui;
GuiApp::GuiApp(int& argc, char * argv[]) : QApplication(argc,argv)
{
    //ctor
    InitResources();
    InitLocale();
    m_settings = new QSettings(QSettings::IniFormat,QSettings::UserScope,MiraOrgStr,MiraNameStr);
    m_mainWindow = new MainWindow();
}

GuiApp::~GuiApp()
{
    //dtor
    delete m_settings;
    delete m_mainWindow;
}
void GuiApp::AddContact()
{

    //Load another project
    //Manually load a workplace for now
    Workplace * miraPrj = new Workplace("Thurston's Mira Groupware");

    Utility * util = new Utility("Files");
    util->setIcon(":/images/utility-files.png");
    miraPrj->appendUtility(util);

    util = new Utility("Calendar");
    util->setIcon(":/images/utility-calendar.png");
    miraPrj->appendUtility(util);

    util = new Utility("Discussion");
    util->setIcon(":/images/utility-discussion.png");
    miraPrj->appendUtility(util);

    util = new Utility("Notes");
    util->setIcon(":/images/utility-notes.png");
    miraPrj->appendUtility(util);

    AddWorkplace(miraPrj);
    setActiveWorkplace(miraPrj);
}

Workplace * GuiApp::getActiveWorkplace()
{
    /*
    This function will return the current workplace that is loaded.  If no workplace is loaded, then it will return
    the workplace saved by the settings.  If no workplace is saved in the settings, then it will return the first
    workplace in the list of user's workplaces.  If the user has no workplace, it will return NULL
    */

    QString qname = m_settings->value("currentWP").toString();
    std::string name = qname.toStdString();
    Workplace * wp = getWorkplace(name);

    if ((wp == NULL) && (!m_workplaces.empty()))
    {
        //we couldn't find the workplace with the given name, but there are loaded workplaces.
        //we'll default to the beginning workplace
        wp = *m_workplaces.begin();
    }
    return wp;
}

Workplace * GuiApp::getWorkplace(std::string name)
{
    for (WorkplaceList::iterator iter = m_workplaces.begin(); iter != m_workplaces.end(); ++iter)
    {
        Workplace * wp = (*iter);
        if (wp->getName() == name)
            return wp;
    }
    return NULL;
}

WorkplaceList GuiApp::getWorkplaces()
{
    return m_workplaces;
}

void GuiApp::AddWorkplace(Workplace * wp)
{
    m_workplaces.push_back(wp);
    emit AddedWorkplace(wp);
}
void GuiApp::LoadWorkplaces()
{

    //Manually load a workplace for now
    Workplace * bluesPrj = new Workplace("Blues Project");

    Utility * util;
    util = new Utility("Files");
    util->setIcon(":/images/utility-files.png");
    bluesPrj->appendUtility(util);

    util = new Utility("Calendar");
    util->setIcon(":/images/utility-calendar.png");
    bluesPrj->appendUtility(util);

    util = new Utility("Discussion");
    util->setIcon(":/images/utility-discussion.png");
    bluesPrj->appendUtility(util);

    util = new Utility("Forms");
    util->setIcon(":/images/utility-forms.png");
    bluesPrj->appendUtility(util);

    util = new Utility("Notes");
    util->setIcon(":/images/utility-notes.png");
    bluesPrj->appendUtility(util);

    util = new Utility("Polls");
    util->setIcon(":/images/utility-polls.png");
    bluesPrj->appendUtility(util);
    AddWorkplace(bluesPrj);

    //Load another project
    //Manually load a workplace for now
    Workplace * miraPrj = new Workplace("Mira Groupware");

    util = new Utility("Files");
    util->setIcon(":/images/utility-files.png");
    miraPrj->appendUtility(util);

    util = new Utility("Calendar");
    util->setIcon(":/images/utility-calendar.png");
    miraPrj->appendUtility(util);

    util = new Utility("Discussion");
    util->setIcon(":/images/utility-discussion.png");
    miraPrj->appendUtility(util);

    util = new Utility("Notes");
    util->setIcon(":/images/utility-notes.png");
    miraPrj->appendUtility(util);

    AddWorkplace(miraPrj);
    //setActiveWorkplace(miraPrj);
}

void GuiApp::InitResources()
{
    //initialize the resources
    //Q_INIT_RESOURCE(Resources);
}

void GuiApp::InitLocale()
{
    //initialize the translation to the user's locale
    QString locale = QLocale::system().name();
    QString langStr = QString::fromAscii("mira_")+locale;
    QTranslator translator;
    translator.load(langStr,QString::fromAscii(":/qms"));
    installTranslator(&translator);
}

void GuiApp::setActiveWorkplace(Workplace * wp,Utility * util)
{
	emit ChangedWorkplace(wp,util);
}

void GuiApp::SendChatMessage(std::string msg)
{
	//send a chat message through the network

	//TODO: For now we are hardcoding a message coming back
	emit ReceiveChatMessage("Bob","It's done");
	emit ReceiveChatError("Network disconnected");
}
