#include "LoginDialog.h"
#include "AsioClient.h"

#include <boost/thread.hpp>

#include <QIcon>

#include <iostream>

using namespace miraclient::gui;

LoginDialog::LoginDialog(QWidget *parent) : QDialog(parent)
{
    setWindowTitle(tr("Connect To Mira Server"));

    resize(400, 200);

    m_mainLayout = new QVBoxLayout(this);

    m_loginGroupBox = new QGroupBox();
    m_loginGroupBox->setTitle("Login Information");

    m_formLayout = new QFormLayout(m_loginGroupBox);

    m_usernameLineEdit = new QLineEdit(m_loginGroupBox);
    connect(m_usernameLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(checkIfComplete()));
    m_formLayout->addRow(tr("&Username:"), m_usernameLineEdit); 

    m_passwordLineEdit = new QLineEdit(m_loginGroupBox);
    m_passwordLineEdit->setEchoMode(QLineEdit::Password);
    connect(m_passwordLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(checkIfComplete()));
    m_formLayout->addRow(tr("&Password:"), m_passwordLineEdit);

    m_serverLineEdit = new QLineEdit(m_loginGroupBox);
    connect(m_serverLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(checkIfComplete()));
    m_formLayout->addRow(tr("&Server:"), m_serverLineEdit);

    m_portLineEdit = new QLineEdit(m_loginGroupBox);
    m_portLineEdit->setText("7351");
    connect(m_portLineEdit, SIGNAL(textChanged(const QString&)), this, SLOT(checkIfComplete()));
    m_formLayout->addRow(tr("&Port:"), m_portLineEdit);

    m_mainLayout->addWidget(m_loginGroupBox);

    m_buttonsLayout = new QFormLayout();
    m_connectButton = new QPushButton(tr("Connect"), this);
    m_connectButton->setEnabled(false);
    m_connectButton->setIcon(QIcon(":/images/transmit_blue.png"));
    connect(m_connectButton, SIGNAL(clicked()), this, SLOT(connectToServer()));
    m_cancelButton  = new QPushButton(tr("Cancel"), this);
    m_cancelButton->setIcon(QIcon(":/images/cancel.png"));
    connect(m_cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
    m_buttonsLayout->addRow(m_connectButton, m_cancelButton);
    m_errorLabel = new QLabel;
    m_buttonsLayout->addRow(m_errorLabel);
    m_buttonsLayout->setFormAlignment(Qt::AlignRight);
    m_mainLayout->addItem(m_buttonsLayout);

    m_mainLayout->addLayout(m_buttonsLayout);
}


void LoginDialog::checkIfComplete()
{
  if ( m_usernameLineEdit->text().isEmpty() || m_passwordLineEdit->text().isEmpty() || m_serverLineEdit->text().isEmpty() || m_portLineEdit->text().isEmpty() )
    m_connectButton->setDisabled(true);
  else
    m_connectButton->setEnabled(true);
}


void LoginDialog::connectToServer()
{
    miraclient::network::asio::AsioClient* client = new miraclient::network::asio::AsioClient();
    if (client)
    {
        boost::thread(boost::bind(&network::asio::AsioClient::connect, client, m_serverLineEdit->text().toStdString(), m_portLineEdit->text().toUInt(), 
                                  m_usernameLineEdit->text().toStdString(), m_passwordLineEdit->text().toStdString()));
    }
}

void LoginDialog::cancel()
{
    reject();
}

void LoginDialog::loginFailed(const QString& error_message)
{
    m_errorLabel->setText("Failed to connect to server: " + error_message);
}
