#include "MainWindow.h"
#include "Version.h"
#include "MiraDefs.h"
#include "LoginDialog.h"
#include "Application.h"
#include "GuiApp.h"

#include <QAction>
#include <QMenuBar>
#include <QMessageBox>

#include <iostream>

#include <QString>

using namespace miraclient::gui;

MainWindow::MainWindow()
{
    //ctor

    setWindowIcon(QIcon(":/images/icon-small.png"));
    setWindowTitle(tr(MiraNameStr)+" "+MiraVersionStr);

    m_workplaceWidget = new WorkplaceWidget(NULL);
    setCentralWidget(m_workplaceWidget);

    createDockWidgets();
    createActions();
    createMenus();
    createConnections();
    LoadSettings();

    // m_loginDialog = NULL;
}

MainWindow::~MainWindow()
{
    //dtor
    delete m_workplaceWidget;
    delete m_exitAction;
    delete m_aboutAction;
}

void MainWindow::setActiveWorkplace(Workplace * wp,Utility * util)
{
	/*
	if (wp != m_workplaceWidget->getWorkplace())
	{
		WorkplaceWidget * oldwp = m_workplaceWidget;
		m_workplaceWidget = new WorkplaceWidget(wp);
		//TODO: setup the new widget with the dimensions of the old one
		setCentralWidget(m_workplaceWidget);
		delete oldwp;
	}
	if (util != NULL)
	{
		m_workplaceWidget->setActiveUtility(util);
	}

	*/
	m_workplaceWidget->setActiveWorkplace(wp,util);
}

void MainWindow::createConnections()
{
    GuiApp * app = (GuiApp *)QApplication::instance();
    connect(m_loginAction, SIGNAL(triggered()), SLOT(login()));
    connect(m_exitAction,SIGNAL(triggered()),this,SLOT(close()));
    connect(m_aboutAction,SIGNAL(triggered()),SLOT(about()));
    connect(m_addContactAction,SIGNAL(triggered()),app,SLOT(AddContact()));
    connect(m_chatFocusAction,SIGNAL(triggered()),m_chatWidget,SLOT(FocusOnChat()));
    connect(m_chatWidget,SIGNAL(sendingMessage(std::string)),app,SLOT(SendChatMessage(std::string)));
    connect(app,SIGNAL(ReceiveChatMessage(std::string,std::string)),m_chatWidget,SLOT(receiveMessage(std::string,std::string)));
    connect(app,SIGNAL(ReceiveChatError(std::string)),m_chatWidget,SLOT(receiveError(std::string)));
    connect(app,SIGNAL(AddedWorkplace(Workplace *)),m_navigationWidget,SLOT(AddWorkplace(Workplace *)));
    connect(m_navigationWidget,SIGNAL(changedWorkplace(Workplace *,Utility *)),app,SLOT(setActiveWorkplace(Workplace *,Utility *)));
    connect(app,SIGNAL(ChangedWorkplace(Workplace *,Utility *)),this,SLOT(setActiveWorkplace(Workplace *,Utility *)));
}

void MainWindow::createActions()
{
    //Exit Action
    m_exitAction = new QAction(tr("E&xit"), this);
    m_exitAction->setShortcut(tr("Ctrl+X"));
    m_exitAction->setStatusTip(tr("Exit Mira Groupware Client Application"));

    //About Action
    m_aboutAction = new QAction(tr("&About"),this);
    m_aboutAction->setShortcut(tr("Ctrl+A"));
    m_aboutAction->setStatusTip(tr("Show information about Mira Groupware"));

    //Login Action
    m_loginAction = new QAction(tr("&Login"),this);
    m_loginAction->setShortcut(tr("Ctrl+L"));
    m_loginAction->setStatusTip(tr("Login to a Mira Server"));

    //Add Contact Action
    m_addContactAction = new QAction(tr("A&dd a Contact"),this);
    m_addContactAction->setShortcut(tr("Ctrl+D"));
    m_addContactAction->setStatusTip(tr("Add a contact"));

    //Add a widget so the chat box can have a shortcut
    m_chatFocusAction = new QAction(tr("Focus Cursor on Chat"),this);
    m_chatFocusAction->setShortcut(tr("Ctrl+H"));
}

void MainWindow::closeEvent(QCloseEvent * event)
{
	SaveSettings();
}
void MainWindow::SaveSettings()
{
	GuiApp * app = (GuiApp *)QApplication::instance();
    //save the settings
    app->m_settings->beginGroup("mainWindow");
    app->m_settings->setValue("maximized",isMaximized());
    //if it is not maximized, save the size and position
    if (!isMaximized())
    {
        app->m_settings->setValue("size",size());
        app->m_settings->setValue("pos",pos());
    }
    app->m_settings->setValue("state",saveState());
    app->m_settings->endGroup();


    app->m_settings->setValue("currentWP", m_workplaceWidget->getName().c_str());
}
void MainWindow::LoadSettings()
{
	GuiApp * app = (GuiApp *)QApplication::instance();
    app->m_settings->beginGroup("mainWindow");
    restoreState(app->m_settings->value("state").toByteArray());
    resize(app->m_settings->value("size",QSize(800,600)).toSize());
    move(app->m_settings->value("pos").toPoint());
    if (app->m_settings->value("maximized",false).toBool())
        setWindowState(Qt::WindowMaximized);

    app->m_settings->endGroup();
}

void MainWindow::createMenus()
{
    //File Menu
    m_fileMenu = menuBar()->addMenu(tr("&File"));
    m_fileMenu->addAction(m_exitAction);
 
    //Login Menu
    m_actionsMenu = menuBar()->addMenu(tr("&Actions"));
    m_actionsMenu->addAction(m_loginAction);

    //View Menu
    //m_viewMenu = createPopupMenu();
    QMenu * popupMenu = createPopupMenu();
    popupMenu->setTitle("&Dock Windows");
    m_viewMenu = menuBar()->addMenu("&View");
    m_viewMenu->addMenu(popupMenu);
    m_viewMenu->addSeparator();
    m_viewMenu->addAction(m_chatFocusAction);


    //Help Menu
    m_helpMenu = menuBar()->addMenu(tr("&Help"));
    m_helpMenu->addAction(m_aboutAction);

    //Contact Menu
    m_contactMenu = menuBar()->addMenu(tr("Contact"));
    m_contactMenu->addAction(m_addContactAction);
}

void MainWindow::about()
{
    QString aboutStr = "<h2>"+tr(MiraNameStr)+" "+tr(MiraVersionStr)+"</h2>";
    aboutStr += "<p><h4>"+tr(MiraCopyrightStr)+"</h4></p>";
    aboutStr += tr("<p>Mira is an Open Source groupware platform which aims to provide a completely free collaboration solution for enterprises and power users.</p><p><a href=\"http://miragroupware.org/\">http://miragroupware.org/</a></p>");
    QMessageBox::about(this,tr("About Mira Groupware"),
            aboutStr);
}

void MainWindow::login()
{
    m_loginDialog = new LoginDialog(this);
    m_loginDialog->exec();
}

void MainWindow::createDockWidgets()
{
    m_navigationWidget = new NavigationDockWidget();
    m_navigationWidget->setObjectName("navDockWidget"); //object name needs to be set for saveState() to work
    addDockWidget(Qt::LeftDockWidgetArea, m_navigationWidget);

    m_contactWidget = new ContactDockWidget();
    m_contactWidget->setObjectName("contact"); //object name needs to be set for saveState() to work
    addDockWidget(Qt::RightDockWidgetArea, m_contactWidget);

    m_chatWidget = new ChatDockWidget();
    m_chatWidget->setObjectName("chatDockWidget"); //object name needs to be set for saveState() to work
    addDockWidget(Qt::RightDockWidgetArea, m_chatWidget);
}

void MainWindow::loginSuccessful()
{
    setWindowTitle(tr(MiraNameStr) + " " + MiraVersionStr + " Connected to: " + Application::get_client()->get_remote_server_address().c_str());
    m_loginDialog->accept();
    delete m_loginDialog;
    m_loginDialog = NULL;
    m_loginAction->setDisabled(true);
}

void MainWindow::loginFailed(const QString& error_message)
{
    m_loginDialog->loginFailed(error_message);
}

void MainWindow::serverConnectionLost()
{
    setWindowTitle(tr(MiraNameStr) + " " + MiraVersionStr);
    m_loginAction->setDisabled(false);
    emit serverConnectionTerminated();
}
