#include "NavigationDockWidget.h"
#include "NavigationTreeWidgetItem.h"
#include "Utility.h"
#include <QList>
using namespace miraclient::gui;

NavigationDockWidget::NavigationDockWidget()
{
    //ctor
    setWindowTitle(tr("Navigation"));
    navigationTree = new QTreeWidget;
    setWidget(navigationTree);
    setupUI();
    navigationTree->expandAll();

    //connect(navigationTree,SIGNAL(itemActivated(QTreeWidgetItem *)),this,SLOT(workplaceActivated(Workplace *)));
    connect(navigationTree,SIGNAL(itemSelectionChanged()), this, SLOT(changeWorkplaceSelection()));

}

NavigationDockWidget::~NavigationDockWidget()
{
    //dtor
    delete navigationTree;
}

void NavigationDockWidget::setupUI()
{
    QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(navigationTree->sizePolicy().hasHeightForWidth());
    navigationTree->setSizePolicy(sizePolicy);
    navigationTree->setMinimumSize(QSize(100, 0));
    navigationTree->headerItem()->setText(0, QString());
    navigationTree->clear();
}

void NavigationDockWidget::AddWorkplace(Workplace * wp)
{
	NavigationTreeWidgetItem * rootItem= new NavigationTreeWidgetItem(navigationTree);
    rootItem->setText(0, wp->getName().c_str());
    rootItem->setWorkplace(wp);

    UtilityList utils = wp->getUtilities();
    for (UtilityList::iterator iter = utils.begin(); iter != utils.end(); ++iter)
    {
        Utility * util = (*iter);
        NavigationTreeWidgetItem * __item = new NavigationTreeWidgetItem(rootItem);
        __item->setText(0, util->getName().c_str());
        __item->setIcon(0, QIcon(util->getIcon().c_str()));
        __item->setUtility(util);
        __item->setWorkplace(wp);
    }
    rootItem->setExpanded(true);
}

void NavigationDockWidget::closeEvent(QCloseEvent * event)
{
    emit closed(false);
}


void NavigationDockWidget::changeWorkplaceSelection()
{
	QList <QTreeWidgetItem *> selectionList = navigationTree->selectedItems();
	NavigationTreeWidgetItem * selElem = static_cast<NavigationTreeWidgetItem * > (selectionList.first());
	emit changedWorkplace(selElem->getWorkplace(),selElem->getUtility());

}
