#include "WorkplaceWidget.h"
#include "Utility.h"
#include <QTabWidget>
#include <QTextEdit>
#include <QTextBrowser>
#include <QMessageBox>

using namespace miraclient::gui;
WorkplaceWidget::WorkplaceWidget(Workplace * workplace)
{
    //ctor
    m_workplace = workplace;
    setupUI();
}

void WorkplaceWidget::setupUI()
{
    QVBoxLayout * m_mainLayout = new QVBoxLayout;
    m_mainWpWidget = new QTabWidget;
    QString lblText;
    m_workplaceLabel = new QLabel;
    m_mainLayout->addWidget(m_workplaceLabel);
    m_mainLayout->addWidget(m_mainWpWidget);
    setLayout(m_mainLayout);
    if (m_workplace == NULL)
    {
        //setup a dummy UI
    	m_workplaceLabel->setText(QString("<h1>%1</h1>").arg(tr("Welcome to Mira Groupware")));
    	QTextBrowser * p = new QTextBrowser();
    	p->insertHtml(tr("check us out at <a href= \"http://www.miragroupware.org\">www.miragroupware.org</a>"));
    	p->setOpenExternalLinks(true);
    	m_mainWpWidget->addTab((static_cast<QWidget *>(p)),tr("Welcome!"));
    	m_tabList.push_back((static_cast<QWidget *>(p)));
    }
    else
    {
    	setActiveWorkplace(m_workplace);
    }
}

std::string WorkplaceWidget::getName()
{
    if (m_workplace != NULL)
        return m_workplace->getName();
    else
        return "";
}

WorkplaceWidget::~WorkplaceWidget()
{
    //dtor
	delete m_mainWpWidget;
}

Workplace * WorkplaceWidget::getWorkplace()
{
	return m_workplace;
}
void WorkplaceWidget::setActiveWorkplace(Workplace * wp,Utility * util)
{
	m_workplace=wp;
	m_workplaceLabel->setText(QString("<h1>%1</h1>").arg(wp->getName().c_str()));

	//Remove all the current utility tabs
	m_mainWpWidget->clear();
	while (!m_tabList.isEmpty())
		delete m_tabList.takeFirst();

	UtilityList utils = m_workplace->getUtilities();
    int indexNum=0;
    for(UtilityList::iterator utilItem = utils.begin(); utilItem !=utils.end(); ++utilItem )
    {
        Utility * util = *utilItem;
        m_utils.insert(util,indexNum);
        indexNum++;
        QWidget * newTab = new QWidget();
        m_tabList.push_back(newTab);
        m_mainWpWidget->addTab(newTab,QIcon(util->getIcon().c_str()),util->getName().c_str());
    }
	setActiveUtility(util);
}

void WorkplaceWidget::setActiveUtility(Utility * util)
{
	if (util != NULL)
	{
	int indexNum=m_utils.value(util);
	(static_cast<QTabWidget *>(m_mainWpWidget))->setCurrentIndex(indexNum);
	}
}
