/*
 * Msg.cpp - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <algorithm>
#include <string>
#include <cstring>
#include <string>
#include <iostream>

#include "Msg.h"
#include "MsgUserAccepted.h"
#include "MsgLoginSuccessful.h"
#include "MsgError.h"

using std::string;
using std::cout;
using std::endl;

namespace miraclient
{

namespace network
{

Msg::Msg(const string& message) : m_message(message)
{
}

void Msg::Parse(string message)
{
    // Don't use Client/Server security tokens for this version.
    // Also security tokens should be taken care at TcpConnection.
    // If right token is not given ignore message.

    // This is going to separate the message string into two string.
    // The token string, and start the message string after the token string.

    Msg* msg = NULL;
    char token_string[3]; // Support a max of 2 characters for Tokens.

    unsigned int pos = message.find(' ');

    // TODO: check that we only get 2 chars for token_string
    if (pos != string::npos)
    {
        strcpy(token_string, message.substr(0, pos).c_str());
        message.erase(0, strlen(token_string)+1); // +1 to remove extra space.
    }
    else
    {
        strcpy(token_string, message.c_str());
        message.clear();
    }

    switch(get_token(token_string))
    {
        case USER_ACCEPTED:
            msg = new MsgUserAccepted(message);
            break;

        case LOGIN_SUCCESSFUL:
            msg = new MsgLoginSuccessful(message);
            break;

        case ERROR:
            msg = new MsgError(message);
            break;

        default:
            msg = NULL;
            break;
    }

    if (msg)
    {
        msg->run();
        delete msg;
    }
}

std::map<const char*, Msg::Token, TokenCompare> Msg::m_token_map = TokenMap();

} // namespace directory

} // namespace miraclient
