/*
 * Field.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2009 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER_FIELD_H
#define __MIRASERVER_FIELD_H

#include <string>
#include <list>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/version.hpp>
#include <boost/serialization/list.hpp>

// Microsoft Visual C++ doesn't provide a cstdint header.
#ifdef _MSC_VER // [
#include <boost/cstdint.hpp>
using boost::uint8_t;
#else
#include <stdint.h>
#endif // _MSC_VER ]

struct Field
{
    static const uint8_t type_int8       = 0x0;    /* int8_t      */
    static const uint8_t type_uint8      = 0x1;    /* uint8_t     */
    static const uint8_t type_int16      = 0x2;    /* int16_t     */
    static const uint8_t type_uint16     = 0x3;    /* uint16_t    */
    static const uint8_t type_int32      = 0x4;    /* int32_t     */
    static const uint8_t type_uint32     = 0x5;    /* uint32_t    */
    static const uint8_t type_int64      = 0x6;    /* int64_t     */
    static const uint8_t type_uint64     = 0x7;    /* uint64_t    */
    static const uint8_t type_float      = 0x8;    /* float       */
    static const uint8_t type_double     = 0x9;    /* double      */
    static const uint8_t type_string     = 0xa;    /* std::string */
    static const uint8_t type_invalid    = 0xff;   /* Means this field is invalid */


    // Makes a list type. std::list< type >
    static uint8_t list(uint8_t type) { return uint8_t(0x10 | type); }
 
    static bool is_list(uint8_t type) { return bool(0x10 & type); }

    template <class Archive>
    void serialize(Archive& ar, const unsigned int /* file_version */)
    {
        ar & BOOST_SERIALIZATION_NVP(type);
        ar & BOOST_SERIALIZATION_NVP(attribute);
        ar & BOOST_SERIALIZATION_NVP(value);
        ar & BOOST_SERIALIZATION_NVP(value_list);
    }

/*
    Field(const uint8_t _type, const std::string& _attribute, const std::string& _value, const std::list<std::string>& _value_list)
    : type(_type), attribute(_attribute), value(_value), value_list(_value_list)
    {}

    Field() : type(type_invalid)
    {}
*/
   
    uint8_t                type;
    std::string            attribute;
    std::string            value;
    std::list<std::string> value_list;
};

#endif // __MIRASERVER__FIELD_H
