/*
 * User.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER_USER_H
#define __MIRASERVER_USER_H

#include <string>
#include <list>
#include <map>

#include <boost/serialization/nvp.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/version.hpp>

#include "Field.h"

namespace miraserver
{

struct User
{
   public:
   	typedef std::list<unsigned int> WorkPlaceListType;
        typedef std::map< std::string, Field > FieldTableType;

        User() : m_id(0) {};

        User(const unsigned int id, const std::string& username, FieldTableType field_table = FieldTableType()) : m_id(id), m_username(username), m_field_table(field_table)
        {}

   	User(const std::string& username, FieldTableType field_table = FieldTableType()) : m_id(0), m_username(username), m_field_table(field_table)
   	{}

   	~User() {};

        unsigned int get_id() const { return m_id; }
        void set_id(const unsigned int id) { m_id = id; }

        std::string get_username() const { return m_username; }
        void set_username(const std::string& username) { m_username = username; }

        // Users with an id of 0 are considered invalid
        bool is_valid() { return bool(m_id); }

        void set_field(const Field& field)
        {
            m_field_table[field.attribute] = field;
        }

        Field get_field(const std::string& attribute) const
        {
            const FieldTableType::const_iterator Iter = m_field_table.find( attribute );
            if ( Iter != m_field_table.end() )
            {
                return Iter->second;
            }
            else
            {
                Field ret_val = { Field::type_invalid, "", "" };
                return ret_val;
            }
        }

   private:
        friend class boost::serialization::access;

        template <class Archive>
        void serialize(Archive& ar, const unsigned int /* file_version */)
        {
    	    ar & BOOST_SERIALIZATION_NVP(m_id);
            ar & BOOST_SERIALIZATION_NVP(m_username);
            ar & BOOST_SERIALIZATION_NVP(m_field_table);
        }

        unsigned int       m_id;
        std::string        m_username;
        FieldTableType     m_field_table;
};

} // namespace miraserver

#endif // __MIRASERVER__USER_H
