/*
 * WorkPlace.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER_WORKPLACE_H
#define __MIRASERVER_WORKPLACE_H

#include <string>
#include <list>
#include <map>

#include <boost/serialization/nvp.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/version.hpp>

#include "Field.h"

namespace miraserver
{

class WorkPlace
{
   public:
   	typedef std::list<unsigned int> UserListType;
        typedef std::map< std::string, Field > FieldTableType;

        WorkPlace() : m_id(0) {}
        WorkPlace(const std::string workplace_name, FieldTableType field_table = FieldTableType()) 
        : m_id(0), m_workplace_name(workplace_name), m_field_table(field_table) 
        {}

   	WorkPlace(const unsigned int id, const std::string workplace_name, FieldTableType field_table = FieldTableType()) 
        : m_id(id), m_workplace_name(workplace_name), m_field_table(field_table)
        {}

   	~WorkPlace() {}

    	unsigned int get_id() const { return m_id; }
        void set_id(const unsigned int id) { m_id = id; }

        std::string get_workplace_name() const { return m_workplace_name; }
        void set_workplace_name(const std::string& workplace_name) { m_workplace_name = workplace_name; }

        // WorkPlaces with an id of 0 are considered invalid.
        bool is_valid() { return bool(m_id); }

        void set_field(const Field& field)
        {
            m_field_table[field.attribute] = field;
        }

        Field get_field(const std::string& attribute) const
        {
            FieldTableType::const_iterator Iter = m_field_table.find( attribute );
            if ( Iter != m_field_table.end() )
            {
                return Iter->second;
            }
            else
            {
                Field ret_val = { Field::type_invalid, "", "" };
                return ret_val;
            }
        }


   private:
        friend class boost::serialization::access;

        template<class Archive>
        void serialize(Archive & ar, const unsigned int /* file_version */)
        {
    	    ar & BOOST_SERIALIZATION_NVP(m_id);
            ar & BOOST_SERIALIZATION_NVP(m_workplace_name);
            ar & BOOST_SERIALIZATION_NVP(m_field_table);
        }

    	unsigned int       m_id;
        std::string        m_workplace_name;
        FieldTableType     m_field_table;
};

} // namespace miraserver

#endif // __MIRASERVER__WORKGPLACE_H
