/*
 * Directory.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER__DIRECTORY_DIRECTORY_H
#define __MIRASERVER__DIRECTORY_DIRECTORY_H

#include <string>
#include <list>
#include <utility>
#include <map>

#include "User.h"
#include "WorkPlace.h"
#include "Resource.h"

namespace miraserver
{

namespace directory
{

// Base class used for directory backends.
class Directory
{
   public:
   	Directory() {}
   	virtual ~Directory() {}

   	virtual unsigned int login(const std::string &username, const std::string &password) = 0;

    	virtual bool add_user(User& user) = 0;
    	virtual bool add_workplace(WorkPlace& workplace) = 0;
        virtual bool add_resource(const Resource& resource) = 0;

    	virtual User find_user(const std::string& username) = 0;
	virtual User find_user(const unsigned int user_id) = 0;
        
        virtual Resource find_resource(const std::string& resource_name) = 0;

        // Update User in backend
        virtual bool update_user(const User& user) = 0;
        // Update WorkPlace in backend
        virtual bool update_workplace(const WorkPlace& workplace) = 0;
        virtual bool update_resource(const Resource& resource) = 0;

    	virtual WorkPlace find_workplace(const std::string& workplace_name) = 0;
    	virtual WorkPlace find_workplace(const unsigned int workplace_id) = 0;


   protected:

   private:
};

} // namespace directory

} // namespace miraserver

#endif // __MIRASERVER__DIRECTORY_DIRECTORY_H
