/*
 * Msg.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER__NETWORK_MSG_H
#define __MIRASERVER__NETWORK_MSG_H

#include <cstring>
#include <string>
#include <map>

#include <iostream>

namespace miraserver
{

namespace network
{

// forward declaration of TcpConnection
class TcpConnection;

struct TokenCompare
{
    // This assumes token characters are upper case.
    inline bool operator()( const char* lhs, const char* rhs ) const
    {
        return strcmp(lhs, rhs) < 0;
    }
};

// Base class for Network Messages.
// The function that parses the raw messages and
// creates the right Msg class is a static member of
// this base class. (static void Parse()).
class Msg
{
    public:
        Msg(TcpConnection* source_connection, const std::string& message);
        virtual ~Msg() {}

        enum Token
        {
            NONE,
// Not sure where it' coming from but the preprocessor var ERROR seems to be defined in CodeBlocks/Windows.
// Work around this by saving the value of ERROR in TMP and then setting it again.
#ifdef ERROR
#define TMP ERROR
#undef ERROR
#endif
            ERROR,
#ifdef TMP
#define ERROR TMP
#endif
            NEW_USER,
            USER_ACCEPTED,
            LOGIN,
            LOGIN_SUCCESS,
            MESSAGE,
            PRIVATE_MESSAGE,
            NEW_GROUP,
            GROUP_ACCEPTED,
            GROUP,
            JOIN_GROUP,
            USER,
            QUERY,
            TOKEN,
            PING,
            PONG
        };


    static void  Parse(TcpConnection*  source_connection, std::string* message);
    virtual void  run() = 0;
        // Return NONE if token is not found.
    static Token get_token(const char* token_string)
        {
            TokenMap::iterator iter;
            iter = m_token_map.find(token_string);
            return iter == m_token_map.end() ? NONE : iter->second;
        }
    static void add_token(Token token, const char* token_string)
        {
            m_token_map.insert(TokenMap::value_type(token_string, token));
        }
   /* static void set_new_acct_crt(const unsigned int nu_acct = 0)
    {
        //If m_new_acct_crt is 0 then anyone can create the new user account
        //If set to 1 then only admin/group manager can create the new account.
        m_new_acct_crt = nu_acct; 
    }
    static bool nu_allowed()
    {
       return (m_new_acct_crt != 1); //New User acct creator 1 means only admin/group manager can create the new acct.
    }*/

    protected:
        typedef std::map<const char*, Token, TokenCompare> TokenMap;

        std::string       m_message;
        TcpConnection*    m_source_connection;

        static TokenMap   m_token_map;
    private:
      // static unsigned int m_new_acct_crt; //This detemines if anyone can create the new
                                            //user account or only admin/group manager.
};

} // namespace network
} // namespace miraserver


#endif // __MIRASERVER__NETWORK_MSG_H
