/*
 * Server.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER__NETWORK_SERVER_H
#define __MIRASERVER__NETWORK_SERVER_H

#include <string>
#include <map>

#include "TcpConnection.h"

namespace miraserver
{

namespace network
{

class Server
{
    public:

        Server() {}
        virtual ~Server() {}

        // This function will take care of accepting new client connections.
        // This function should have a loop inside of it so further calls to
        // it are not needed.
        virtual void accept() = 0;

        // receiver is user_id of the user the message is being sent to.
        // returns true if user was found and message was delivered.
        virtual bool send(const unsigned int user_id, const std::string &buffer) = 0;

        // This function returns a reference to the connection associated with username.
        virtual TcpConnection* get_connection(const unsigned int user_id)
        {
            ConnectionListType::iterator iter = m_connection_list.find(user_id);
            return iter != m_connection_list.end() ? iter->second : NULL;
        }

        virtual void add_connection(const unsigned int user_id, TcpConnection* tcp_connection)
        {
            m_connection_list.insert(ConnectionListType::value_type(user_id, tcp_connection));
        }

        virtual void remove_connection(const unsigned int user_id)
        {
            m_connection_list.erase(user_id);
        }

        // returns true if there is a TcpConnection associated with username.
        virtual bool is_connected(const unsigned int user_id)
        {
            return (m_connection_list.find(user_id) != m_connection_list.end());
        }

        // Makes the server run its loop.
        virtual void run() = 0;

    protected:
        typedef std::map<unsigned int, TcpConnection* >  ConnectionListType;

        ConnectionListType  m_connection_list; // List of connections associated by username

    private:
};

} // namespace network

} // namespace miraserver

#endif // __MIRASERVER__NETWORK_SERVER_H
