/*
 * AsioServer.h - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __MIRASERVER__NETWORK_ASIOSERVER_H
#define __MIRASERVER__NETWORK_ASIOSERVER_H

#include <boost/asio.hpp>
#include <boost/thread.hpp>
#include <boost/bind.hpp>

#include "TcpConnection.h"
#include "Server.h"

using namespace boost;

namespace miraserver
{

namespace network
{

class AsioServer : public Server
{
   public:
      typedef asio::ip::tcp::socket SocketType;

      AsioServer(unsigned int port);

      void accept();
      void handle_accept(SocketType* asio_socket);

      bool send(const unsigned intuser_id, const std::string &buffer);

      // Used mainly as function pointer by TcpConnection.
      static bool send(TcpConnection& tcp_connection);
      static bool receive(TcpConnection &tcp_connection);
      static void close_socket(void* socket);

      void run();


   private:

      asio::io_service m_io_service;
      asio::ip::tcp::acceptor m_acceptor;
};

} // namespace network

} // namespace miraserver

#endif // __MIRASERVER__NETWORK_ASIOSERVER_H
