/*
 * MsgJoinGroup.h
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGJOINGROUP_H
#define __MIRASERVER__NETWORK_MSGJOINGROUP_H

#include <string>

#include "Msg.h"
#include "tools.h"
#include "MsgQuery.h"

namespace miraserver
{

namespace network
{
class MsgJoinGroup : public Msg
{
    public:
        MsgJoinGroup(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run()
        {
            //Splitting message into fields
            vector<std::string> message_field = Split(m_message, " ");
            
            //Checking on minimum fields
            if(message_field.size() >= 1)//excluding token, stripped at Msg::Parse
            {
                //Checking if the Group exist
                WorkPlace workgroup = Application::get_directory().find_workplace(message_field.at(0));
                if(workgroup.is_valid())
                {
                    User user = Application::get_directory().find_user(m_source_connection->get_user_id());
                    //Adding group(id) to user's workplace_list
                    Field field = user.get_field(ATTRIBUTE_WORKPLACE_LIST);
                    if(field.type == Field::type_invalid)
                    {
                        field.type      = Field::list(Field::type_uint16);
                        field.attribute = ATTRIBUTE_WORKPLACE_LIST;
                        field.value     = " ";
                        field.value_list = std::list<std::string>(1, IntToString(workgroup.get_id()));
                    }    
                    else
                    {
                        field.value_list.push_back(IntToString(workgroup.get_id())); 
                    }
                    user.set_field(field);
                    Application::get_directory().update_user(user);

                    //Adding user(id) to Group's user_list
                    field = workgroup.get_field(ATTRIBUTE_USER_LIST);
                    field.value_list.push_back(IntToString(user.get_id()));
                    
                    workgroup.set_field(field);
                    Application::get_directory().update_workplace(workgroup);
              
                    string message = "G * GroupName=";
                    message += message_field.at(0);
                    Msg *msg = new MsgQuery(m_source_connection, message);
                    msg->run();
                    delete msg;
                    //m_source_connection->send(return_message);
                }
                else
                {
                    string error_message = "E JG Group ";
                    error_message += message_field.at(0) + " doesn't exist";
                    m_source_connection->send(error_message);
                }
            }
            else
            {
                m_source_connection->send("E JG Invalid Format ");
            }

        };
       
        ~MsgJoinGroup() {}

    private:
};

}

}

#endif // __MIRASERVER__NETWORK_MSGJOINGROUP_H
