/*
 * MsgLogin.h
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifdef __MIRASERVER__NETWORK_MSGLOGIN_H
#define TEMP __MIRASERVER__NETWORK_MSGLOGIN_H
#undef __MIRASERVER__NETWORK_MSGLOGIN_H
#endif

#ifndef __MIRASERVER__NETWORK_MSGLOGIN_H
#define __MIRASERVER__NETWORK_MSGLOGIN_H

#include <string>
#include <sstream>
#include <vector>

#include "Msg.h"
#include "Application.h"
#include "User.h"
#include "Field.h"
#include "field_defs.h"
#include "tools.h"
#include "MsgError.h"

using namespace std;

namespace miraserver
{
namespace network
{
class MsgLogin : public Msg
{
    public:
        MsgLogin(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run()
        {
            //Splitting message into fields
            vector<std::string> message_field = Split(m_message, " ");

            //Checking on minimum fields
            if (message_field.size() >= 2)//excluding token, stripped at Msg::Parse
            {
                string username = message_field.at(0);
                string password = message_field.at(1);
                unsigned int uid = Application::get_directory().login(username, password);
                if (uid)
                {
                    //Marking tcp_connection login.
                    m_source_connection->set_user_id(uid);
                    Application::get_server().add_connection(uid, m_source_connection);
                    m_source_connection->send("LS");
                
                    // If the user has messages then send them to user.
                    User user = Application::get_directory().find_user(uid);
                    Field field = user.get_field(ATTRIBUTE_MESSAGE_LIST);
                    if((field.type != Field::type_invalid) && (!field.value_list.empty()))
                    {
                        std::list<std::string>::iterator it;
                        for(it=field.value_list.begin();it!=field.value_list.end();)
                        {
                            m_source_connection->send(*it);
                            //Deleting message from user's message_list
                            it = field.value_list.erase(it);
                        }   
                        user.set_field(field);
                        Application::get_directory().update_user(user);        
                    }    
                }
                else
                {
                    m_source_connection->send("E LS Bad Credentials");
                }
            }   
            else
            { 
                m_source_connection->send("E L Invalid Format");
            }    
        };

        ~MsgLogin() {}

    private:
};

}

}

#endif // __MIRASERVER__NETWORK_MSGLOGIN_H

#ifdef TEMP
#define __MIRASERVER__NETWORK_MSGLOGIN_H TEMP
#endif
