/*
 * MsgMessage.h 
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGMESSAGE_H
#define __MIRASERVER__NETWORK_MSGMESSAGE_H

#include <string>
#include <vector>
#include <ctime>

#include "Msg.h"
#include "tools.h"
#include "User.h"
#include "WorkPlace.h"
#include "TcpConnection.h"
#include "field_defs.h"

namespace miraserver
{

namespace network
{
class MsgMessage : public Msg
{
    public:
        MsgMessage(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run() 
        {
            string message;

            //Splitting message into fields
            vector<std::string> message_field = Split(m_message, " ");

            //Checking on minimum fields  
            if(message_field.size() >= 2)//excluding token, stripped at Msg::Parse
            {
                WorkPlace group = Application::get_directory().find_workplace(message_field.at(0));

                //Checking if the group exist.
                if(group.is_valid())
                {
                    Field field = group.get_field(ATTRIBUTE_USER_LIST);

                    //Ckecking if the group user_list is not empty
                    if((field.type != Field::type_invalid) && (!field.value_list.empty()))
                    {
                        std::list<std::string>::iterator it;

                        //Sending message to  all the users in the group
                        for(it=field.value_list.begin();it!=field.value_list.end();it++)
                        {
                            User user = Application::get_directory().find_user(StringToInt(*it));
                            TcpConnection *tcp_connection = Application::get_server().get_connection(user.get_id());

                            message  = "M " +  user.get_username();
                            message += " " + message_field.at(0);
                            message += " " + message_field.at(1);
                            message += " " + IntToString(time(NULL));

                            //Checking if user is online
                            if(tcp_connection != NULL)
                                tcp_connection->send(message);
                            else
                            { 
                                //If user is offline then save the message in it's message_list
                                Field msg_field = user.get_field(ATTRIBUTE_MESSAGE_LIST);
                                //If the message_list doesn't exist then create one
                                if(msg_field.type == Field::type_invalid)
                                {
                                    msg_field.type       = Field::list(Field::type_uint16);
                                    msg_field.attribute  = ATTRIBUTE_MESSAGE_LIST;  
                                    msg_field.value      = " ";
                                    msg_field.value_list = std::list<std::string>(1, message);
                                }
                                else
                                { 
                                    //Add message to the existing message_list
                                    msg_field.value_list.push_back(message);
                                }//else

                                //Update user
                                user.set_field(msg_field);
                                Application::get_directory().update_user(user);

                            }//else
                        }//for
                    }//if
                    else
                    {
                        string error_message = "E M group ";
                        error_message += message_field.at(0) + " doesn't have any users";
                        m_source_connection->send(error_message);
                    } 
                }
                else
                {
                    string error_message = "E M group ";
                    error_message += message_field.at(0) + " does not exist";
                    m_source_connection->send(error_message);
                }
            }
            else
            {
                m_source_connection->send("E M Invalid Format");
            } 
        };

        ~MsgMessage() {}

    private:
};

}

}

#endif // __MIRASERVER__NETWORK_MSGMESSAGE_H
