/*
 * MsgNewGroup.h 
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGNEWGROUP_H
#define __MIRASERVER__NETWORK_MSGNEWGROUP_H

#include <string>
#include <sstream>
#include <vector>

#include "Msg.h"
#include "PlainTextDirectory.h"
#include "Application.h"
#include "WorkPlace.h"
#include "User.h"
#include "field_defs.h"
#include "tools.h"

namespace miraserver
{

namespace network
{
class MsgNewGroup : public Msg
{
    public:
        MsgNewGroup(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run() 
        {
                
            //Splitting message into fields
            vector<std::string> message_field = Split(m_message, " ");
            
            if(message_field.size() >= 2) //excluding token, stripped at MSG::Parse
            {
                //Checking if the Group Name already exist.
                if(Application::get_directory().find_workplace(message_field.at(0)).is_valid())
                {
                    string error_message = "E NG Group ";
                    error_message += message_field.at(0) + " exist";
                    m_source_connection->send(error_message);
                }
                else
                {
                    WorkPlace workgroup(message_field.at(0));
                    User user = Application::get_directory().find_user(message_field.at(1));
                    if(user.is_valid())
                    {   
                        //Adding Group to Database
                        Field field = {Field::list(Field::type_uint16), ATTRIBUTE_USER_LIST," ", std::list<std::string>(1, IntToString(user.get_id()))};
                        workgroup.set_field(field);

                        Application::get_directory().add_workplace(workgroup);
                    
                        //Add group(id) to users worlplace list
                        field = user.get_field(ATTRIBUTE_WORKPLACE_LIST);
                        if(field.type == Field::type_invalid)
                        {
                            field.type       = Field::list(Field::type_uint16); 
                            field.attribute  = ATTRIBUTE_WORKPLACE_LIST;
                            field.value      = " "; 
                            field.value_list = std::list<std::string>(1, IntToString(workgroup.get_id()));
                        }
                        else
                        {
                            field.value_list.push_back(IntToString(workgroup.get_id()));    
                        }

                        user.set_field(field);
                        Application::get_directory().update_user(user);

                        string return_message = "GA ";
                        return_message += message_field.at(0);
                        m_source_connection->send(return_message);
                    }          
                    else
                    {   
                        m_source_connection->send("E NG owner does not exist");
                    }
                      
                }   
            }     
            else
            {
                m_source_connection->send("E NG Invalid Format");
            }
        };   
 
        ~MsgNewGroup() {}

    private:
};

}

}

#endif // __MIRASERVER__NETWORK_MSGNEWGROUP_H
