/*
 * MsgNewUser.h 
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGNEWUSER_H
#define __MIRASERVER__NETWORK_MSGNEWUSER_H

#include <string>
#include <sstream>
#include <vector>
#include <iostream>

#include "Application.h"
#include "field_defs.h"
#include "Msg.h"
#include "Crypto.h"
#include "tools.h"

using namespace std;

namespace miraserver
{
namespace network
{
class MsgNewUser : public Msg
{
    public:
        MsgNewUser(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run()
        { 
              
            // Splitting message into fields. 
            vector<std::string> message_field = Split(m_message, " ");

            if(message_field.size() >= 4) //excluding token, stripped at Msg::parse
            {
                // Checking if the username already exist
                if(Application::get_directory().find_user(message_field.at(0)).is_valid())
                {
                    string error_message = "E User ";
                    error_message += message_field.at(0) + " exist";
                    m_source_connection->send(error_message);
                }
                else
                {
                    User user(message_field.at(0));
                    
                    Field field = {Field::type_string, ATTRIBUTE_EMAIL, message_field.at(1), std::list<std::string>()};
                    user.set_field(field);
                    
                    // TODO: If client sends encrypted password then
                    // we don't need to encrypt password here. 
   
                    field.attribute = ATTRIBUTE_PASSWORD;
                    field.value = Crypto::EncryptPassword(message_field.at(2));
                    user.set_field(field);

                    field.attribute = ATTRIBUTE_NAME;
                    field.value = message_field.at(3);
                    user.set_field(field);
                    
                    //Optional attributes in NU message    
                    if(message_field.size() > 4)
                    {
                        field.attribute = ATTRIBUTE_LAST_NAME;
                        field.value = message_field.at(4);
                        user.set_field(field);
                    }   
                
                    Application::get_directory().add_user(user);

                    //After role model is developed this will change.
                    //For Now:If NU account is created with out loggin then m_source_id=0 and hence user is not associated with tcp_connection.
                    string return_message = "UA ";
                    return_message += IntToString(user.get_id()) + " " + message_field.at(0);
                    m_source_connection->send(return_message);
                }//else
            }//if
            else
            {
                m_source_connection->send("E NU Invalid Format");
                cout << "E NU Invalid Format" << endl;
            }
        };

        ~MsgNewUser() {}

    private:
};

}//namespace network

}//namespace mira


#endif // __MIRASERVER__NETWORK_MSGNEWUSER_H
