/*
 * MsgPong.h
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGPONG_H
#define __MIRASERVER__NETWORK_MSGPONG_H

#include <string>
#include <sstream>
#include <vector>

#include "Msg.h"
#include "tools.h"
#include "MsgError.h"

namespace miraserver
{

namespace network
{
class MsgPong : public Msg
{
    public:
        MsgPong(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run()
        {
            string *message = NULL;
            Msg    *msg = NULL;
            
            //Splitting message into fields
            vector<std::string> message_field = Split(m_message, " ");

            //Checking on minimum fields
            if(message_field.size() >= 1 )
            {
                message = new string("PO ");
                *message += message_field.at(0); 
                m_source_connection->send(*message);
                delete message;

            }
            else
            {
                message = new string("E P Invalid Format");
                msg = new MsgError(m_source_connection, *message);
            }    
            if(msg)
            {
                msg->run();
                delete msg;
                delete message;
            }
        };

        ~MsgPong() {}

    private:
};

}

}

#endif // __MIRASERVER__NETWORK_MSGPONG_H
