/*
 * MsgPrivateMessage.h
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGPRIVATEMESSAGE_H
#define __MIRASERVER__NETWORK_MSGPRIVATEMESSAGE_H

#include <string>
#include <vector>

#include "Msg.h"
#include "tools.h"
#include "User.h"
#include "WorkPlace.h"
#include "TcpConnection.h"
#include "field_defs.h"

namespace miraserver
{

namespace network
{
class MsgPrivateMessage : public Msg
{
    public:
        MsgPrivateMessage(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {}

        virtual void run() 
        {

            //Splitting message into fields
            vector<std::string> message_field = Split(m_message, " ");

            //Checking on minimum fields  
            if(message_field.size() >= 2)//excluding token, stripped at Msg::Parse
            {
                User user = Application::get_directory().find_user(message_field.at(0));
                //Checking if the user exist.
                if(user.is_valid())
                {
                    TcpConnection *tcp_connection = Application::get_server().get_connection(user.get_id());

                    string message   = "PM ";
                    message += Application::get_directory().find_user(m_source_connection->get_user_id()).get_username();
                    message += " " + message_field.at(0);
                    message += " " + message_field.at(1);
                    message += " " + IntToString(time(NULL));
                    

                    //Checking if the user is online
                    if(tcp_connection != NULL)
                        tcp_connection->send(message);
                    else
                    { 
                        //If user is offline then save the message in it's message_list
                        Field field = user.get_field(ATTRIBUTE_MESSAGE_LIST);
                        //If the message_list doesn't exist then create one
                        if(field.type == Field::type_invalid)
                        {
                            field.type       = Field::list(Field::type_uint16);
                            field.attribute  = ATTRIBUTE_MESSAGE_LIST;  
                            field.value      = " ";
                            field.value_list = std::list<std::string>(1, message);
                        }
                        else
                        {
                            //Add message to the existing message_list
                            field.value_list.push_back(message);
                        }

                        //Update user
                        user.set_field(field);
                        Application::get_directory().update_user(user);

                    }//else
                }//if
                else
                {
                    string error_message = "E PM user ";
                    error_message += message_field.at(0) + " doesn't exist";
                    m_source_connection->send(error_message);
                }
            }//if
            else
            {
                m_source_connection->send("E PM Invalid Format");
            } 
        };

        ~MsgPrivateMessage() {}

    private:
};

}

}

#endif // __MIRASERVER__NETWORK_MSGPRIVATEMESSAGE_H
