/*
 * MsgQuery.h
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef __MIRASERVER__NETWORK_MSGQUERY_H
#define __MIRASERVER__NETWORK_MSGQUERY_H

#include <string>
#include <sstream>
#include <vector>
#include <map>

#include "Msg.h"
#include "tools.h"
#include "TcpConnection.h"

namespace miraserver
{

namespace network
{
class MsgQuery : public Msg
{
    public:
        MsgQuery(TcpConnection* source_connection, const std::string& message) : Msg(source_connection, message)
        {
            //If the m_field_map size is non-zero then it is initialised before.
            if(m_field_map.size() == 0) 
                init_field_map();
        }

        virtual void run()
        {
             std::string message;

             //Splitting message into fields
             std::vector<std::string> message_field = Split(m_message, " ");

             //Checking on minimum fields
             if(message_field.size() >= 2) //excluding token, stripped at MSG::Parse
             {
                 switch(get_field(message_field.at(0)))
                 {
                     case U:
                         message = "U ";
                         message += message_field.at(1);
                         if(message_field.size() > 2)//Optional field
                             message += " " + message_field.at(2);

                         query_users(message); // place-holder
                         break;
                     case G:
                         message = "G ";
                         message += message_field.at(1);
                         if(message_field.size() > 2)//Optional field
                         {   
                             message += " " + message_field.at(2);
                         }
                         query_groups(message);
                         break;

                     default:
                         m_source_connection->send("E Q Incorrect first field.\n Expected: USERS\\GROUPS");
                         break; 
                 }
             }
             else
             {
                 m_source_connection->send("E Q Invalid Format");
             }
        };

        ~MsgQuery() {}

    private:
        enum FirstField
        {
            N,
            U,
            G
        };

        static void init_field_map()
        {
            m_field_map["U"] = U;
            m_field_map["G"] = G;
        }

        static FirstField get_field(const std::string field)
        {
            FieldMap::iterator it;
            it = m_field_map.find(field);
            return it == m_field_map.end()? N : it->second;
        }

        void query_groups(const std::string& query);
        void query_users(const std::string& query);

        typedef std::map<std::string, FirstField> FieldMap;
        static FieldMap m_field_map;

};

}

}

#endif // __MIRASERVER__NETWORK_MSGQUERY_H
