/*
 * ConfigParser.cpp - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <cstdlib>
#include <string>
#include <fstream>
#include <iostream>
#include <sstream>

#include "ConfigParser.h"

using std::string;
using std::getline;
using std::ifstream;
using std::cout;
using std::endl;
using std::stringstream;

namespace miraserver
{

ConfigParser::ConfigParser(const std::string& _ConfigFile) : ConfigMap(), ConfigFile(_ConfigFile), M_parsed(false)
{
}

void ConfigParser::ParseConfigFile()
{
   ifstream ifsConfigFile(ConfigFile.c_str()); // Opening Configuration file
   if (!ifsConfigFile.is_open())
   {
   	cout << "ConfigParser Error: Could not open " << ConfigFile << endl;
   	exit(0);
   }

   unsigned int intCurrentLine = 0;
   string strCurrentLine;
   while(getline(ifsConfigFile, strCurrentLine))
   {
   	intCurrentLine++; // start with line 1.

   	if (strCurrentLine == "")
   	   continue;

   	// replace this with a strip function.
   	while(strCurrentLine[0] == ' ')
   	{
   	   strCurrentLine = strCurrentLine.substr(1);
   	}

   	// Comment line. Ignore.
   	if (strCurrentLine[0] == '#' || strCurrentLine[0] == '/')
   	   continue;

   	if (!IsItem(strCurrentLine.substr(0, strCurrentLine.find(" "))))
   	{
   	   // handle error "Unrecognizable configuration"
   	   cout << "Line " << intCurrentLine << ": Unrecognizable configuration." << endl;
   	   continue;
   	}

   	string strItem = strCurrentLine.substr(0, strCurrentLine.find(" "));
   	strCurrentLine = strCurrentLine.substr(strItem.length()+1);

   	// replace this with a strip function.
        while(strCurrentLine[0] == ' ')
        {
           strCurrentLine = strCurrentLine.substr(1);
        }

   	if (strCurrentLine[0] != '=')
   	{
   	   // handle error "Expected '=' Token."
   	   cout << "Line " << intCurrentLine << ": Expected \'=\' Token." << endl;
   	   continue;
   	}
   	strCurrentLine = strCurrentLine.substr(1);

        // replace this with a strip function.
        while(strCurrentLine[0] == ' ')
        {
           strCurrentLine = strCurrentLine.substr(1);
        }

   	if (strCurrentLine[0] != '\"')
   	{
   	   // handle error "Expected '\"' Token."
   	   cout << "Line " << intCurrentLine << ": Expected \'\"\' Token (0x01)." << endl;
   	   continue;
   	}

   	// Format is ITEM = "CONFIGURATION"
   	string strConfiguration = strCurrentLine.substr(1);
   	if (strConfiguration.find('\"') != string::npos)
   	{
   	   for (unsigned int i = 0; i < strConfiguration.length(); i++)
   	   {
   	   	if ('\"' == strConfiguration[i] && '\\' != strConfiguration[i-1])
   	   	   strConfiguration = strConfiguration.substr(0, i);
   	   }
   	}
   	else
  	{
   	   // handle error "Expected '\"' Token"
   	   cout << "Line " << intCurrentLine << ": Expected \'\"\' Token (0x02)." << endl;
   	   continue;
   	}
   	SetConfiguration(strItem, strConfiguration);
   }

   ifsConfigFile.close();
   M_parsed = true;
}


bool ConfigParser::IsItem(const string& Item)
{
   ConfigMapType::iterator iterConfigMap = ConfigMap.find(Item);

   if (ConfigMap.end() == iterConfigMap)
   	return false;

   // else
   return true;
}

std::string ConfigParser::GetConfiguration(const string& Item)
{
   ConfigMapType::iterator iterConfigMap = ConfigMap.find(Item);

   if (ConfigMap.end() == iterConfigMap)
    return "";
   // else
   return iterConfigMap->second;
}

int ConfigParser::GetConfigurationAsInt(const string& Item)
{
    stringstream ss(GetConfiguration(Item));

    int r_int = 0;
    ss >> r_int;

    return r_int;
}

void ConfigParser::SetConfiguration(const string& Item, const string& Configuration)
{
   ConfigMapType::iterator iterConfigMap = ConfigMap.find(Item);

   if (ConfigMap.end() != iterConfigMap)
   	iterConfigMap->second = Configuration;
}

bool ConfigParser::InsertItem(const string& Item, const string& Configuration)
{
   return ConfigMap.insert(ConfigMapType::value_type(Item,Configuration)).second;
}

} // namespace miraserver

