/*
 * PlainTextDirectory.cpp - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <string>
#include <fstream>
#include <cstdio>

#include <boost/serialization/map.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>

#include "Directory.h"
#include "PlainTextDirectory.h"
#include "Crypto.h"

using std::cout;
using std::endl;

namespace miraserver
{

namespace directory
{

unsigned int PlainTextDirectory::login(const std::string &username, const std::string &password)
{
    User l_user = find_user(username);
    if (l_user.is_valid())
    {
        Field field = l_user.get_field("password");
        if (field.type != Field::type_invalid)
            if (Crypto::MatchPassword(password, field.value))
                return l_user.get_id();
    }

    return 0; // invalid id
}

void PlainTextDirectory::load()
{
    // open the archive
    std::ifstream ifs(m_filename.c_str());
    assert(ifs.good());
    boost::archive::xml_iarchive ia(ifs);

    // restore the schedule from the archive
    ia >> boost::serialization::make_nvp("Directory", *this);
}

void PlainTextDirectory::save()
{
    std::ofstream ofs(m_filename.c_str());
    assert(ofs.good());
    boost::archive::xml_oarchive oa(ofs);
    oa << boost::serialization::make_nvp("Directory", *this);
}

} // namespace directory

} // namespace miraserver
