/*
 * main.cpp - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <string>
#include <list>

#include <boost/thread.hpp>
#include <boost/bind.hpp>

#include "PlainTextDirectory.h"
#include "AsioServer.h"
#include "Application.h"
#include "ConfigParser.h"
#include "Msg.h"
#include "Directory.h"
#include "User.h"
#include "WorkPlace.h"
#include "TcpConnection.h"
#include "field_defs.h"
#include "tools.h"
#include "Resource.h"

using namespace miraserver;
using namespace directory;
using namespace network;
using namespace std;

using boost::thread;

// Login Credentials for debugging purposes are:
// User: clsk
// Pass: MyPassword123
int main()
{
    try
    {
        Application::set_configuration(new ConfigParser);

        Application::get_configuration().InsertItem("PORT");
        Application::get_configuration().InsertItem("NETWORK_THREAD_COUNT");
	Application::get_configuration().InsertItem("NEW_ACCOUNT_CREATOR");
        // Application::get_configuration().InsertItem("ASIO_INPUT_BUFFER_SIZE");

        Application::get_configuration().ParseConfigFile();

        Application::set_directory(new PlainTextDirectory("Directory.xml"));
        Application::set_server(new AsioServer(Application::get_configuration().GetConfigurationAsInt("PORT")));

/*              
        User clsk("clsk");

        Field field = { Field::type_string, ATTRIBUTE_PASSWORD, "xutJTzPg5d1cf7dd8ff17bc8f4722ffe7ccee0e7", std::list<std::string>() };

        //Field field = { Field::type_string, ATTRIBUTE_PASSWORD, "xutJTzPg44d0e339e34264727bdc96787a517a90", std::list<std::string>() };
       
        clsk.set_field(field);

        field.attribute = ATTRIBUTE_EMAIL; field.value = "alan.alvarez@us.army.mil";
        clsk.set_field(field);

        field.attribute = ATTRIBUTE_NAME; field.value = "Alan";
        clsk.set_field(field);

        field.attribute = ATTRIBUTE_LAST_NAME; field.value = "Alvarez";
        clsk.set_field(field);
        
        // Directory::add_user() sets the user id.
        Application::get_directory().add_user(clsk);

        WorkPlace mira("Mira");
        field.type = Field::list(Field::type_uint16);
        field.attribute = ATTRIBUTE_USER_LIST;
        field.value = "";
        field.value_list = std::list<std::string>(1, IntToString(clsk.get_id()));
        mira.set_field(field);

        Application::get_directory().add_workplace(mira);

        field.attribute = ATTRIBUTE_WORKPLACE_LIST;
        field.value_list = std::list<std::string>(1, IntToString(mira.get_id()));
        clsk.set_field(field);
        Application::get_directory().update_user(clsk);


        Resource server_new_user("server_new_user");
        server_new_user.give_read_access(clsk.get_id());
        server_new_user.give_write_access(clsk.get_id());
        Application::get_directory().add_resource(server_new_user);*/

    }
    catch (string error)
    {
        cout << error << endl;
    }

    // register tokens
    Msg::add_token(Msg::NONE, "");
    Msg::add_token(Msg::ERROR, "E"); // Error
    Msg::add_token(Msg::LOGIN, "L");
    Msg::add_token(Msg::NEW_USER, "NU");
    Msg::add_token(Msg::USER_ACCEPTED, "UA");
    Msg::add_token(Msg::MESSAGE, "M");
    Msg::add_token(Msg::PRIVATE_MESSAGE, "PM");
    Msg::add_token(Msg::NEW_GROUP, "NG");
    Msg::add_token(Msg::GROUP_ACCEPTED, "GA");
    Msg::add_token(Msg::GROUP, "G");
    Msg::add_token(Msg::JOIN_GROUP, "JG");
    Msg::add_token(Msg::USER, "U");
    Msg::add_token(Msg::QUERY, "Q");
    Msg::add_token(Msg::TOKEN, "T");
    Msg::add_token(Msg::PING, "PI");
    Msg::add_token(Msg::PONG, "PO");
    // TODO: Need to add rest of messages.

    unsigned short thread_count = Application::get_configuration().GetConfigurationAsInt("NETWORK_THREAD_COUNT");
    TcpConnection::add_thread(thread_count); // add threads to thread pool.

    /*unsigned int new_acct_crt = Application::get_configuration().GetConfigurationAsInt("NEW_ACCOUNT_CREATOR");
    std::cout << "new_acct_crt:" << new_acct_crt << std::endl;
    Msg::set_new_acct_crt(new_acct_crt); //sets whether anyone or only admin/manager can create new account.
    */
    
    Application::get_server().run();

    // Delete objects inside Application to make sure they are terminated correctly.
    delete &Application::get_configuration();
    delete &Application::get_server();
    delete &Application::get_directory();
    return 0;
}
