/*
 * Msg.cpp - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <algorithm>
#include <string>
#include <cstring>
#include <string>
#include <iostream>

#include "Msg.h"
#include "MsgLogin.h"
#include "MsgNewUser.h"
#include "MsgNewGroup.h"
#include "MsgJoinGroup.h"
#include "MsgQuery.h"
#include "MsgMessage.h"
#include "MsgPrivateMessage.h"
#include "MsgPing.h"
//#include "MsgPong.h"


using std::string;
using std::cout;
using std::endl;

namespace miraserver
{

namespace network
{

//unsigned int Msg::m_new_acct_crt = 0;

Msg::Msg(TcpConnection* source_connection, const string& message) : m_message(message), m_source_connection(source_connection)
{
}

void Msg::Parse(TcpConnection* source_connection, string* message)
{
    // Don't use Client/Server security tokens for this version.
    // Also security tokens should be taken care at TcpConnection.
    // If right token is not given ignore message.

    // This is only going to parse incoming messages from client into two strings.
    // The token string, and the message string starting after the token string.

    Msg* msg = NULL;
    char token_string[3]; // Support a max of 2 characters for Tokens.

    unsigned int pos = message->find(' ');

    if (pos != string::npos)
    {
        strcpy(token_string, message->substr(0, pos).c_str());
        message->erase(0, strlen(token_string)+1); // +1 to remove extra space.
    }
    else
    {
        strcpy(token_string, message->c_str());
        message->clear();
    }

    switch(get_token(token_string))
    {
        case LOGIN:
             msg = new MsgLogin(source_connection, *message);
             break;

        case NEW_USER:
             msg = new MsgNewUser(source_connection, *message);
             break;

        case NEW_GROUP:
             msg = new MsgNewGroup(source_connection, *message);
             break;
        
        case JOIN_GROUP:
             msg = new MsgJoinGroup(source_connection, *message);
             break;

        case QUERY:
             msg = new MsgQuery(source_connection, *message);
             break;
  
        case MESSAGE:
             msg = new MsgMessage(source_connection, *message);
             break;

        case PRIVATE_MESSAGE:
             msg = new MsgPrivateMessage(source_connection, *message);
             break;

        case PING:
             msg = new MsgPing(source_connection, *message);
             break;

        case PONG:
             //msg = new MsgPong(source_connection, *message);
             break;

        default:
            string error_msg = " Invalid Token";
            error_msg.insert(0, token_string);
            source_connection->send(error_msg);
            break;
    }

    if (msg)
    {
        msg->run();
        delete msg;
    }

    delete message; // created at TcpConnection::handle_read() and at derived messages run()
}

std::map<const char*, Msg::Token, TokenCompare> Msg::m_token_map = TokenMap();

} // namespace network 

} // namespace miraserver
