/*
 * AsioServer.cpp - by Alan Alvarez <alan.alvarez@us.army.mil>
 *
 * Mira GroupWare
 * Copyright (C) 2007-2008 Mira GroupWare Dev Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <boost/asio.hpp>
#include <boost/thread/thread.hpp>
#include <boost/bind.hpp>

#include <vector>
#include <string>
#include <iostream>

#include "TcpConnection.h"
#include "AsioServer.h"

using namespace boost;

using asio::ip::tcp;

using boost::thread;
using boost::bind;

using std::cout;
using std::endl;

namespace miraserver
{

namespace network
{

AsioServer::AsioServer(unsigned int port) : Server(), m_io_service(), m_acceptor(m_io_service,
                                                                       tcp::endpoint(tcp::v4(), port))
{
    TcpConnection::set_send_pointer((bool (*)(TcpConnection& tcp_connection))&AsioServer::send);
    TcpConnection::set_receive_pointer((bool (*)(TcpConnection& tcp_connection))&AsioServer::receive);
    TcpConnection::set_close_pointer((void (*)(void* socket))&AsioServer::close_socket);
}

void AsioServer::accept()
{
    // TcpConnection's Destructor will take care of delete(ing) this socket object.
    SocketType* asio_socket = new SocketType(m_io_service);
    m_acceptor.async_accept(*asio_socket, bind(&AsioServer::handle_accept, this, asio_socket));
}


void AsioServer::handle_accept(SocketType* asio_socket)
{
    TcpConnection *tcp_connection = new TcpConnection(static_cast<void*>(asio_socket));


    asio_socket->async_read_some(boost::asio::buffer(tcp_connection->get_input_buffer(), tcp_connection->input_buffer_size()),
                                 bind(&TcpConnection::handle_read, tcp_connection,
                                      boost::asio::placeholders::bytes_transferred));

    accept();

    SocketType::endpoint_type l_endpoint = asio_socket->remote_endpoint();
    cout << "Accepted connection from: " << l_endpoint.address().to_string() << endl;
}



bool AsioServer::send(const unsigned int user_id, const std::string& buffer)
{
    ConnectionListType::iterator iter = m_connection_list.find(user_id);
    if (iter == m_connection_list.end())
        return false;

    iter->second->send(buffer);

    return true;
}

bool AsioServer::send(TcpConnection& tcp_connection)
{
    boost::asio::write(*(static_cast<SocketType*>(tcp_connection.get_socket())), boost::asio::buffer(tcp_connection.get_output_buffer(),
                                                                                                     tcp_connection.output_buffer_size()+1),
                       boost::asio::transfer_all());


    return true;
}

bool AsioServer::receive(TcpConnection& tcp_connection)
{
    static_cast<SocketType*>(tcp_connection.get_socket())->async_read_some(boost::asio::buffer(tcp_connection.get_input_buffer(),
                                                                                               tcp_connection.input_buffer_size()),
                                                                           bind(&TcpConnection::handle_read, &tcp_connection,
                                                                           boost::asio::placeholders::bytes_transferred));
     return true;
}

void AsioServer::close_socket(void* socket)
{
     SocketType* l_socket = static_cast<SocketType*>(socket);
     l_socket->close();
     delete l_socket;
}

void AsioServer::run()
{
    accept();

    m_io_service.run();
}

} // namespace network

} // namespace miraserver
